@@ -1,6 +1,6 @@
 ;;; nyacc/lang/c99/parser.scm - C parser execution
 
-;; Copyright (C) 2015-2019 Matthew R. Wette
+;; Copyright (C) 2015-2020 Matthew R. Wette
 ;;
 ;; This library is free software; you can redistribute it and/or
 ;; modify it under the terms of the GNU Lesser General Public
@@ -364,14 +364,13 @@
 
       (define (run-parse)
 	(let ((info (fluid-ref *info*)))
-	  (raw-parser (lexer #:mode 'decl #:show-incs (cpi-shinc info))
+	  (raw-parser (lexer #:mode 'code #:show-incs (cpi-shinc info))
 		      #:debug (cpi-debug info))))
       
       (let ((bol #t)		 ; begin-of-line condition
 	    (suppress #f)	 ; parsing cpp expanded text (kludge?)
 	    (ppxs (list 'keep))	 ; CPP execution state stack
 	    (info (fluid-ref *info*))	; info shared w/ parser
-	    ;;(brlev 0)			; brace level
 	    (x-def? (cond ((procedure? xdef?) xdef?)
 			  ((eq? xdef? #t) (lambda (n m) #t))
 			  (else def-xdef?))))
@@ -626,6 +625,8 @@
 		      => (lambda (t) (cons t name)))
 		     ((typename? name)
 		      (cons t-typename name))
+		     ((string=? name "_Pragma")
+		      (assc-$ (finish-pragma)))
 		     (else
 		      (cons t-ident name))))))
 	       ((read-c-num ch) => assc-$)
@@ -722,7 +723,8 @@
 		    (show-incs #f)	; show include files
 		    (return-defs #f)	; return (values tree defs)
 		    (debug #f))		; debug
-  (let ((info (make-cpi debug show-incs cpp-defs (cons "." inc-dirs) inc-help)))
+  (let ((info
+	 (make-cpi debug show-incs cpp-defs (cons "." inc-dirs) inc-help)))
     (set-cpi-ptl! info (cons tyns (cpi-ptl info)))
     (with-fluids ((*info* info)
 		  (*input-stack* '()))
@@ -738,7 +740,7 @@
 	    (lambda (key fmt . args) (apply throw 'c99-error fmt args))))
 	(lambda (key fmt . args)
 	  (report-error fmt args)
-	  #f)))))
+	  (if return-defs (values #f '()) #f))))))
 
 ;; === expr parser ====================
 
