@@ -182,6 +182,14 @@
 	(ppx/p rval)
 	(ppx rval)))
 
+  (define (ptr-declr? sx)
+    (memq (sx-tag sx) '(ptr-declr abs-ptr-declr)))
+
+  (define (protect-ptr declr)
+    (if (ptr-declr? declr) (sf "("))
+    (ppx declr)
+    (if (ptr-declr? declr) (sf ")")))
+
   ;; now ((comment xxx) (attributes "aaa;yyy;zzz"))
   (define (pp-attr attr)
     (string-join
@@ -337,8 +345,13 @@
 	   (else (ppx (car pair)))))
 	expr-list))
 
+      ((stmt-expr (block-item-list . ,items))
+       (sf "({\n") (push-il) (for-each ppx items) (pop-il) (sf "})"))
+
       ((udecl . ,rest)
        (ppx `(decl . ,rest)))
+      ((decl (@ . ,attr))		; GNU extension
+       (sf ";"))
       ((decl (@ . ,attr) ,decl-spec-list)
        (ppx decl-spec-list) (sf ";") (comm+nl attr))
       ((decl (@ . ,attr) ,decl-spec-list ,init-declr-list)
@@ -362,6 +375,8 @@
 	    ((type-qual) (sf "~A" (sx-ref (car dsl) 1)))
 	    ((fctn-spec) (sf "~A" (sx-ref (car dsl) 1)))
 	    ((type-spec) (ppx (car dsl)))
+	    ((typeof-expr typeof-type)
+	     (sf "typeof(") (ppx (sx-ref (car dsl) 1)) (sf ") "))
 	    (else (sf "[?:~S]" (car dsl))))
 	  (if (pair? (cdr dsl)) (sf " ")))
 	dsl))
@@ -386,6 +401,7 @@
       ((comp-declr ,declr) (ppx declr))
       ((param-declr ,declr ,item2) (ppx declr) (ppx item2))
       ((param-declr ,declr) (ppx declr))
+      ((param-declr))
 
       ((bit-field ,ident ,expr)
        (ppx ident) (sf " : ") (ppx expr))
@@ -431,6 +447,11 @@
        (for-each ppx defns)
        (pop-il) (sf "}"))
 
+      ((enum-defn (@ . ,attr) (ident ,name) (attribute-list . ,attrs) ,expr)
+       (sf "~A " name) (ppx (sx-ref tree 2))
+       (sf " = ") (ppx expr) (sf ",") (comm+nl attr))
+      ((enum-defn (@ . ,attr) (ident ,name) (attribute-list . ,attrs))
+       (sf "~A " name) (ppx (sx-ref tree 2)) (sf ",") (comm+nl attr))
       ((enum-defn (@ . ,attr) (ident ,name) ,expr)
        (sf "~A = " name) (ppx expr) (sf ",") (comm+nl attr))
       ((enum-defn (@ . ,attr) (ident ,name))
@@ -473,18 +494,27 @@
 	(lambda (dsl)
 	  (case (sx-tag (car dsl))
 	    ((type-qual) (sf "~A" (sx-ref (car dsl) 1)))
+	    ((typeof-expr typeof-type)
+	     (sf "typeof(") (ppx (sx-ref (car dsl) 1)) (sf ") "))
 	    (else (sf "[?:~S]" (car dsl))))
 	  (if (pair? (cdr dsl)) (sf " ")))
 	tql))
 
-      ((array-of ,dir-declr ,arg)
-       (ppx dir-declr) (sf "[") (ppx arg) (sf "]"))
-      ((array-of ,dir-declr)
-       (ppx dir-declr) (sf "[]"))
+      ((ary-declr ,declr ,arg)
+       (protect-ptr declr) (sf "[") (ppx arg) (sf "]"))
+      ((ary-declr ,declr)
+       (protect-ptr declr) (sf "[]"))
       ;; MORE TO GO
+
+      ((abs-ary-declr ,arg1)
+       (sf "[") (ppx arg1) (sf "]"))
+      ((abs-ary-declr ,arg1 ,arg2)
+       (sf "[") (ppx arg1) (ppx arg2) (sf "]"))
+      ((abs-ary-declr)
+       (sf "[]"))
       
-      ((ftn-declr ,dir-declr ,param-list)
-       (ppx dir-declr) (sf "(") (ppx param-list) (sf ")"))
+      ((ftn-declr ,declr ,param-list)
+       (protect-ptr declr) (sf "(") (ppx param-list) (sf ")"))
 
       ((type-name ,spec-qual-list ,abs-declr)
        (ppx spec-qual-list) (ppx abs-declr))
@@ -617,6 +647,47 @@
        (ppx expr1) (sf "; ") (ppx expr2) (sf "; ") (ppx expr3)
        (sf ") ") (ppx stmt))
 
+      ;; asm - parser does not preserve specifiers
+      ((asm-expr (@ . ,attr) ,pat ,outs ,ins ,clobs ,gotos)
+       (if (assq-ref attr 'goto) (sf "asm goto (") (sf "asm ("))
+       (ppx pat) (ppx outs) (ppx ins) (ppx clobs) (ppx gotos) (sf ")"))
+      ((asm-expr (@ . ,attr) ,pat ,outs ,ins ,clobs)
+       (if (assq-ref attr 'volatile) (sf "asm volatile (") (sf "asm ("))
+       (ppx pat) (ppx outs) (ppx ins) (ppx clobs) (sf ")"))
+      ((asm-expr (@ . ,attr) ,pat ,outs ,ins)
+       (if (assq-ref attr 'volatile) (sf "asm volatile (") (sf "asm ("))
+       (ppx pat) (ppx outs) (ppx ins) (sf ")"))
+      ((asm-expr (@ . ,attr) ,pat ,outs)
+       (if (assq-ref attr 'volatile) (sf "asm volatile (") (sf "asm ("))
+       (ppx pat) (ppx outs) (sf ")"))
+      ((asm-expr (@ . ,attr) ,pat)
+       (if (assq-ref attr 'volatile) (sf "asm volatile (") (sf "asm ("))
+       (ppx pat) (sf ")"))
+      ((asm-outputs . ,elts)
+       (sf ": ")
+       (pair-for-each
+	(lambda (pair) (ppx (car pair)) (if (pair? (cdr pair)) (sf ", ")))
+	elts))
+      ((asm-inputs . ,elts)
+       (sf ": ")
+       (pair-for-each
+	(lambda (pair) (ppx (car pair)) (if (pair? (cdr pair)) (sf ", ")))
+	elts))
+      ((asm-clobbers . ,elts)
+       (sf ": ")
+       (pair-for-each
+	(lambda (pair) (ppx (car pair)) (if (pair? (cdr pair)) (sf ", ")))
+	elts))
+      ((asm-operand (ident ,name) ,str ,arg)
+       (sf "[~A] " name) (ppx str) (sf " (") (ppx arg) (sf ")"))
+      ((asm-operand ,str ,arg)
+       (ppx str) (sf " (") (ppx arg) (sf ")"))
+      ((asm-gotos . ,elts)
+       (sf ": ")
+       (pair-for-each
+	(lambda (pair) (ppx (car pair)) (if (pair? (cdr pair)) (sf ", ")))
+	elts))
+       
       ;; jump-statement
       ((goto ,where)
        (pop-il)			; unindent
@@ -633,15 +704,16 @@
       ((trans-unit . ,items)
        (pair-for-each
 	(lambda (pair)
-	  (let ((this (car pair)) (next (and (pair? (cdr pair)) (cadr pair))))
+	  (let ((this (car pair)) (this-tag (caar pair))
+		(next-tag (and (pair? (cdr pair)) (caadr pair))))
 	    (ppx this)
-	    (cond ;; add blank line if next is different or fctn defn
-	     ((not next))
-	     ((eqv? (sx-tag this) (sx-tag next)))
-	     ((eqv? (sx-tag this) 'comment))
-	     ((eqv? (sx-tag next) 'comment) (sf "\n"))
-	     ((not (eqv? (sx-tag this) (sx-tag next))) (sf "\n"))
-	     ((eqv? (sx-tag next) 'fctn-defn) (sf "\n")))))
+	    (cond ;; heuristics for adding blank lines:
+	     ((not next-tag))
+	     ((eqv? this-tag 'comment))
+	     ((eqv? next-tag 'fctn-defn) (sf "\n"))
+	     ((eqv? this-tag next-tag))
+	     ((eqv? next-tag 'comment) (sf "\n"))
+	     (else (sf "\n")))))
 	items))
 
       ((fctn-defn . ,rest) ;; but not yet (knr-fctn-defn)
@@ -654,12 +726,17 @@
 	 (sf " ")
 	 (ppx compd-stmt)))
 
-      ((ptr-declr . ,rest)
-       (ppx (sx-ref tree 1)) (ppx (sx-ref tree 2)))
+      ((ptr-declr ,ptr ,dcl)
+       (ppx ptr) (ppx dcl))
       
-      ((ftn-declr . ,rest)
-       (ppx (sx-ref tree 1))	; direct-declarator
-       (sf "(") (ppx (sx-ref tree 2)) (sf ")"))
+      ((abs-ptr-declr ,ptr)
+       (ppx ptr))
+       
+      ((ftn-declr ,dcl ,params)
+       (ppx dcl) (sf "(") (ppx params) (sf ")"))
+
+      ((abs-ftn-declr ,params)
+       (sf "(") (ppx params) (sf ")"))
 
       ((param-list . ,params)
        (pair-for-each
