@@ -22,6 +22,7 @@
 	    c99-std-help
 	    get-sys-cpp-defs get-sys-inc-dirs split-cpp-defs 
 	    get-gcc-cpp-defs get-gcc-inc-dirs ;; <= deprecated
+	    remove-comments
 	    remove-inc-trees
 	    merge-inc-trees!
 	    move-attributes attrl->attrs attrs->attrl extract-attr
@@ -359,6 +360,23 @@
 
 ;; --- random stuff 
 
+;; @deffn {Procedure} remove-comments sx0 => sx1
+;; Remove comments from the parse tree @var{sx0} generated by @code{parse-c99}.
+;; @end deffn
+(define (remove-comments sx0)
+  (if(not (pair? sx0)) sx0
+     (cons (car sx0)
+	   (let loop ((tail (cdr sx0)))
+	     (cond
+	      ((null? tail)
+	       '())
+	      ((not (pair? (car tail)))
+	       (cons (car tail) (remove-comments (cdr tail))))
+	      ((eq? 'comment (caar tail))
+	       (loop (cdr tail)))
+	      (else
+	       (cons (remove-comments (car tail)) (loop (cdr tail)))))))))
+
 ;; @deffn {Procedure} elifify tree => tree
 ;; This procedure will find patterns of
 ;; @example
